
Morph subclass: #UiTreeViewport
	instanceVariableNames: 'rows columns selectionColor selectedIndex selectionMode indent verticalHeader horizontalHeader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 14:34:26.522'!
addRowFor: aModelIndex

	| row |
	aModelIndex column = 1
		ifTrue: ["Create new row."
			rows
				ifEmpty: [row := rows add: UiViewRowInfo new]
				ifNotEmpty: [row := rows add: UiViewRowInfo new after: (rows
					reverseDetect: [:r |
						"sibling row"
						r parentModelIndex = aModelIndex parent
						"parent row"
						or: [r firstModelIndex = aModelIndex parent]])]]
		ifFalse: ["Find the row."
			row := rows
				reverseDetect: [:r | r parentModelIndex = aModelIndex parent]].
	
	^ row! !

!UiTreeViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/10/2010 15:08:50.852'!
columnAt: index

	^ columns at: index! !

!UiTreeViewport methodsFor: 'geometry' stamp: 'mt 10/10/2010 13:03:00.575'!
extent: aPoint

	super extent: aPoint.
	self resizedToWidth: aPoint x height: aPoint y.! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:00:05.566'!
horizontalHeader

	^ horizontalHeader! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:00:11.666'!
horizontalHeader: aMorph

	horizontalHeader := aMorph.! !

!UiTreeViewport methodsFor: 'signals' stamp: 'mt 10/10/2010 13:02:41.787'!
resizedToWidth: newWidth height: newHeight

	self emit.! !

!UiTreeViewport methodsFor: 'rows/columns - to info' stamp: 'mt 10/10/2010 15:08:43.412'!
rowAt: index

	^ rows at: index! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 13:59:08.486'!
setHeaderItem: aMorph atColumn: anInteger

	(columns at: anInteger) headerItem: aMorph.! !

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 13:58:39.037'!
setHeaderItem: aMorph atRow: anInteger

	"No custom row labels yet."! !

!UiTreeViewport methodsFor: 'layouting' stamp: 'mt 10/10/2010 14:27:30.741'!
updateLayout

	| heights widths |
	columns do: [:column | column updateSize].
	rows do: [:row | row updateSize].

	heights := rows collect: [:r | r height].
	widths := columns collect: [:c | c width].
	
	columns do: [:column | column layoutItemsWithHeights: heights].
	rows do: [:row | row layoutItemsWithWidths: widths].
	
	columns first items do: [:item |
		item left: self left + (item valueOfProperty: #indent)].

	"Relayout changed header items."
	self horizontalHeader ifNotNil: [:hh |
		hh removeAllMorphs.
		columns do: [:colInfo | hh addMorphBack: colInfo headerItem]].
	self verticalHeader ifNotNil: [:vh |
		vh removeAllMorphs.
		rows do: [:rowInfo | vh addMorphBack: rowInfo headerItem]].! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 13:59:56.367'!
verticalHeader

	^ verticalHeader! !

!UiTreeViewport methodsFor: 'accessing' stamp: 'mt 10/10/2010 13:59:50.126'!
verticalHeader: aMorph

	verticalHeader := aMorph.! !

Object subclass: #UiViewInfo
	instanceVariableNames: 'size items resizeMode headerItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 12:46:28.423'!
headerItem

	^ headerItem! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:07:16.803'!
headerItem: aMorph

	aMorph
		color: (Color gray: 0.85);
		borderStyle: (BorderStyle raised width: 1).
		
	headerItem := aMorph.! !

!UiViewInfo methodsFor: 'initialization' stamp: 'mt 10/10/2010 14:14:48.475'!
initialize

	super initialize.
	
	items := OrderedCollection new.
	resizeMode := #shrinkWrap.
	
	self headerItem: (Morph new extent: 5@5).! !

!UiViewInfo methodsFor: 'geometry' stamp: 'mt 10/10/2010 12:50:22.763'!
resize: aNumber

	size := aNumber.! !

ScrollPane subclass: #UiTreeView
	instanceVariableNames: 'itemDelegates verticalHeader horizontalHeader vHeaderScroller hHeaderScroller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 13:53:09.012'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item |
	
	1 to: (self model rowCountFor: aModelIndex) do: [:row |
		1 to: (self model columnCountFor: aModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aModelIndex.
			item := (self itemDelegateFor: index) itemFor: index.
			self viewport addMorph: item atIndex: index.
			aBoolean ifTrue: [item fadeIn]]].! !

!UiTreeView methodsFor: 'geometry' stamp: 'mt 10/10/2010 15:13:49.001'!
hLeftoverScrollRange

	scroller hasSubmorphs ifFalse:[^0].
	^ (self viewport width - scroller width + 5) max: 0! !

!UiTreeView methodsFor: 'scrolling' stamp: 'mt 10/10/2010 13:46:48.63'!
hScrollBarValue: scrollValue

	super hScrollBarValue: scrollValue.
	hHeaderScroller offset: scroller offset x @ 0.! !

!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/10/2010 14:34:51.957'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; width: self scrollBarThickness; changeTableLayout; listDirection: #topToBottom.
	horizontalHeader := Morph new color: Color transparent; height: self scrollBarThickness; changeTableLayout; listDirection: #leftToRight.
	vHeaderScroller := TransformMorph new color: Color transparent.
	hHeaderScroller := TransformMorph new color: Color transparent.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: UiTreeViewport new.
	self connect: self viewport signal: #doubleClicked: toSelector: #toggleIndex:.
	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	itemDelegates at: #default put: UiSimpleItemDelegate new.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller addMorph: horizontalHeader.

	self
		"addMorph: vHeaderScroller;"
		addMorph: hHeaderScroller.
	
	self resizeScroller.! !

!UiTreeView methodsFor: 'geometry' stamp: 'mt 10/10/2010 14:57:20.328'!
innerBounds

	| inner |
	inner := super innerBounds.
	
	vHeaderScroller owner ifNotNil: [
		inner := inner insetBy: (verticalHeader width @ 0 corner: 0@0)].

	hHeaderScroller owner ifNotNil: [
		inner := inner insetBy: (0 @ horizontalHeader height corner: 0@0)].

	^ inner! !

!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/10/2010 10:42:28.991'!
model: aModel

	super model: aModel.
	self reset.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 14:15:33.809'!
reset
	self viewport clear.
	
	self fillViewportFor: nil animated: false.
	self resetHeaderItems.
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 14:17:03.592'!
resetHeaderItems

	| index item |	
	"1 to: (self model rowCountFor: nil) do: [:row |
		index := self model indexRow: row column: 0 parent: nil.
		item := (self itemDelegateFor: index) itemFor: index.
		item width: verticalHeader width.
		self viewport setHeaderItem: item atRow: row]."
	
	1 to: (self model columnCountFor: nil) do: [:col |
		index := self model indexRow: 0 column: col parent: nil.
		item := (self itemDelegateFor: index) itemFor: index.
		horizontalHeader height: (horizontalHeader height max: item height).
		self viewport setHeaderItem: item atColumn: col].! !

!UiTreeView methodsFor: 'geometry' stamp: 'mt 10/10/2010 14:42:58.971'!
resizeScroller

	super resizeScroller.
	
	hHeaderScroller bounds: (scroller left @ self top
		extent: (scroller width @ horizontalHeader height)).
	vHeaderScroller bounds: (self left @ scroller top
		extent: (verticalHeader width @ scroller height)).! !

!UiTreeView methodsFor: 'geometry' stamp: 'mt 10/10/2010 13:11:49.345'!
scrollBarThickness
	"Includes border"

	^ Preferences scrollBarsNarrow
			ifTrue: [10]
			ifFalse: [14].! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 12:04:23.588'!
toggleIndex: aModelIndex

	| row |
	
	row := self viewport rowFor: aModelIndex.
	
	row isExpanded
		ifTrue: [
			row items first setProperty: #isExpanded toValue: false.
			self viewport collapseRow: row]
		ifFalse: [
			row items first setProperty: #isExpanded toValue: true.
			self
				fillViewportFor: row firstModelIndex
				animated: true].
	
	self viewport
		updateLayout;
		updateExtent.
		
	self updateScrollBars.! !

!UiTreeView methodsFor: 'scrolling' stamp: 'mt 10/10/2010 12:03:46.492'!
updateScrollBars

	self
		resizeScrollBars;
		resizeScroller;
		hideOrShowScrollBars;
		hSetScrollDelta;
		vSetScrollDelta.! !

!UiTreeView methodsFor: 'geometry' stamp: 'mt 10/10/2010 15:10:56.555'!
vLeftoverScrollRange

	scroller hasSubmorphs ifFalse:[^0].
	^ (self viewport height - scroller height + 5) max: 0! !

!UiTreeView methodsFor: 'scrolling' stamp: 'mt 10/10/2010 13:46:57.085'!
vScrollBarValue: scrollValue

	super vScrollBarValue: scrollValue.
	vHeaderScroller offset: 0 @ scroller offset y.! !

!UiDirectoryModel methodsFor: 'accessing - data' stamp: 'mt 10/10/2010 14:52:32.637'!
iconFor: aModelIndex

	aModelIndex row = 0
		ifTrue: [^ nil].

	aModelIndex column = 1
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) isDirectory
			ifTrue: [UiSilkIcons folderIcon]
			ifFalse: [nil "UiSilkIcons pageWhiteIcon"]].
	
	aModelIndex column = 3
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) modificationDateAndTime year = 2010
			ifTrue: [UiSilkIcons newIcon]
			ifFalse: []].
		
	^ nil! !

!UiDirectoryModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:50:53.383'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	(aRowNumber = 0 or: [aColumnNumber = 0])
		ifTrue: [^ (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
			model: self;
			parent: aParentModelIndex].

	^ (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex;
		setProperty: #entry toValue: ((aParentModelIndex
			ifNil: [self root]
			ifNotNil: [:index | (index valueOfProperty: #entry) asFileDirectory]) entries at: aRowNumber)! !

!UiDirectoryModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 15:14:22.181'!
root

	^ FileDirectory on: 'C:'! !

!UiDirectoryModel methodsFor: 'accessing - data' stamp: 'mt 10/10/2010 14:52:41.794'!
textFor: aModelIndex

	aModelIndex row = 0
		ifTrue: [
			aModelIndex column = 1 ifTrue: [^ 'Name'].
			aModelIndex column = 2 ifTrue: [^ 'Size'].
			aModelIndex column = 3 ifTrue: [^ 'Modification Date and Time']].


	aModelIndex column = 1
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) name asText].
		
	aModelIndex column = 2
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) fileSize asString asText].
		
	aModelIndex column = 3
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) modificationDateAndTime asString asText].
		
	^ nil! !

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 12:51:44.846'!
height

	^ size! !

!UiViewRowInfo methodsFor: 'initialization' stamp: 'mt 10/10/2010 12:51:55.334'!
initialize

	super initialize.
	self resize: Preferences standardListFont height.! !

!UiViewRowInfo methodsFor: 'geometry' stamp: 'mt 10/10/2010 14:13:06.245'!
optimalSize
	"Return the height of the largest item."
	
	| max |
	max := 0.
	items do: [:item | max := max max: item height].
	max := max max: self headerItem height.
	^ max! !

!UiViewRowInfo methodsFor: 'geometry' stamp: 'mt 10/10/2010 12:51:17.078'!
resize: anInteger

	super resize: anInteger.
	self headerItem height: anInteger.! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:19:12.733'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	| index |
	index := (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex.
	
	"Header indices."
	(aRowNumber = 0 or: [aColumnNumber = 0])
		ifTrue: [^ index].
	
	aParentModelIndex ifNil: [
		index setProperty: #package toValue: (PackageOrganizer default packages at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		index
			setProperty: #systemCategory
			toValue: ((aParentModelIndex valueOfProperty: #package) systemCategories at: aRowNumber).
		index
			setProperty: #systemCategoryPrefix
			toValue: (aParentModelIndex valueOfProperty: #package) systemCategoryPrefix.
		^ index].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		index
			setProperty: #class
			toValue: ((SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory))
				at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		class := Smalltalk at: (aParentModelIndex valueOfProperty: #class).
		index
			setProperty: #organization
			toValue: class organization.
		index
			setProperty: #protocol
			toValue: (class organization categories at: aRowNumber).
		^ index].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		index
			setProperty: #method
			toValue: ((organization listAtCategoryNamed: protocol) at: aRowNumber).
		^ index].! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/10/2010 14:13:30.029'!
rowCountFor: aParentModelIndex
	
	aParentModelIndex ifNil: [
		^ PackageOrganizer default packages size].
	
	aParentModelIndex column ~= 1 ifTrue: [^ 0].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		^ (aParentModelIndex valueOfProperty: #package) systemCategories size].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		^ (SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory)) size].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		^ (Smalltalk at: (aParentModelIndex valueOfProperty: #class)) organization categories size].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		^ (organization listAtCategoryNamed: protocol) size].
	
	^ 0! !

!UiSmalltalkModel methodsFor: 'accessing - data' stamp: 'mt 10/10/2010 14:13:20.263'!
textFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	"Horizontal headers."
	aModelIndex row = 0 ifTrue: [
		col = 1 ifTrue: [^ 'Name'].
		col = 2 ifTrue: [^ 'Size'].
		col = 3 ifTrue: [^ 'Type']].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ p name].
		col = 2 ifTrue: [^ ('<font color="#555555">', p systemCategories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">package</font>' asHtmlText]].
	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
		
		col = 2 ifTrue: [^ ('<font color="#555555">', (SystemOrganization listAtCategoryNamed: cat) size asString, '</font>') asHtmlText].
		
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		col = 1 ifTrue: [^ cat].
		col = 3 ifTrue: [^ '<font color="#555555">system category</font>' asHtmlText]].
	
	class ifNotNil: [:c |
		col = 1 ifTrue: [^ c].
		col = 2 ifTrue: [^ ('<font color="#555555">', (Smalltalk at: c) organization categories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">class</font>' asHtmlText]].

	protocol ifNotNil: [:p |
		| organization |
		organization := aModelIndex valueOfProperty: #organization.
		col = 1 ifTrue: [^ p].
		col = 2 ifTrue: [^ ('<font color="#555555">', (organization listAtCategoryNamed: p) size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">protocol</font>' asHtmlText]].
	
	method ifNotNil: [:m |
		col = 1 ifTrue: [^ '#', m].
		col = 3 ifTrue: [^ '<font color="#555555">method</font>' asHtmlText]].
		
	^ nil! !

!UiViewColumnInfo methodsFor: 'initialization' stamp: 'mt 10/10/2010 12:52:07.405'!
initialize

	super initialize.
	self resize: 150.! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 10/10/2010 14:12:58.892'!
optimalSize
	"Return the width of the largest item."
	
	| max |
	max := 0.
	items do: [:item | max := max max: (item width + (item valueOfProperty: #indent))].
	max := max max: self headerItem width.
	^ max! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 10/10/2010 12:51:08.192'!
resize: anInteger

	super resize: anInteger.
	self headerItem width: anInteger.! !

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/10/2010 12:52:12.794'!
width

	^ size! !
