
Morph subclass: #UiTreeViewport
	instanceVariableNames: 'rows columns selectionColor selectedIndex selectionMode indent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeViewport methodsFor: 'add/remove' stamp: 'mt 10/10/2010 10:23:19.658'!
addRowFor: aModelIndex

	| row |
	aModelIndex column = 1
		ifTrue: ["Create new row."
			rows
				ifEmpty: [row := rows add: UiViewRowInfo new]
				ifNotEmpty: [row := rows add: UiViewRowInfo new after: (rows
					reverseDetect: [:r |
						"sibling row"
						r parentModelIndex = aModelIndex parent
						"parent row"
						or: [r firstModelIndex = aModelIndex parent]])]]
		ifFalse: ["Find the row."
			row := rows
				reverseDetect: [:r | r parentModelIndex = aModelIndex parent]].
	
	^ row! !

!UiTreeViewport methodsFor: 'layouting' stamp: 'mt 10/9/2010 20:09:25.932'!
updateLayout

	| heights widths |
	columns do: [:column | column updateSize].
	rows do: [:row | row updateSize].

	heights := rows collect: [:r | r height].
	widths := columns collect: [:c | c width].
	
	columns do: [:column | column layoutItemsWithHeights: heights].
	rows do: [:row | row layoutItemsWithWidths: widths].
	
	columns first items do: [:item |
		item left: self left + (item valueOfProperty: #indent)]! !

Object subclass: #UiViewInfo
	instanceVariableNames: 'size items resizeMode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiViewInfo methodsFor: 'initialization' stamp: 'mt 10/9/2010 18:09:48.599'!
initialize

	super initialize.
	
	items := OrderedCollection new.
	resizeMode := #shrinkWrap.! !

!UiViewInfo methodsFor: 'geometry' stamp: 'mt 10/9/2010 18:18:29.095'!
resize: aNumber

	size := aNumber.
	self resized: aNumber.! !

!UiViewInfo methodsFor: 'signals' stamp: 'mt 10/9/2010 18:11:36.975'!
resized: newSize

	self emit.! !

ScrollPane subclass: #UiTreeView
	instanceVariableNames: 'itemDelegates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:59:16.566'!
fillViewportFor: aModelIndex animated: aBoolean

	| index item delegate |
	delegate := self itemDelegateFor: aModelIndex.
	
	1 to: (self model rowCountFor: aModelIndex) do: [:row |
		1 to: (self model columnCountFor: aModelIndex) do: [:col |
			index := self model indexRow: row column: col parent: aModelIndex.
			item := delegate itemFor: index.
			self viewport addMorph: item atIndex: index.
			aBoolean ifTrue: [item fadeIn]]].! !

!UiTreeView methodsFor: 'initialization' stamp: 'mt 10/9/2010 10:54:00.948'!
initialize

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: UiTreeViewport new.
	self connect: self viewport signal: #doubleClicked: toSelector: #toggleIndex:.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	itemDelegates at: #default put: UiSimpleItemDelegate new.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:42:28.991'!
model: aModel

	super model: aModel.
	self reset.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:59:36.167'!
reset
	self viewport clear.
	
	self fillViewportFor: nil animated: false.	
	
	self viewport
		updateLayout;
		updateExtent.! !

!UiTreeView methodsFor: 'as yet unclassified' stamp: 'mt 10/10/2010 10:59:30.202'!
toggleIndex: aModelIndex

	| row |
	
	row := self viewport rowFor: aModelIndex.
	
	row isExpanded
		ifTrue: [
			row items first setProperty: #isExpanded toValue: false.
			self viewport collapseRow: row]
		ifFalse: [
			row items first setProperty: #isExpanded toValue: true.
			self
				fillViewportFor: row firstModelIndex
				animated: true].
	
	self viewport
		updateLayout;
		updateExtent.! !

!UiDirectoryModel methodsFor: 'as yet unclassified' stamp: 'mt 10/8/2010 14:01:00.129'!
hasChildrenFor: aModelIndex

	^ aModelIndex entry isDirectory
		and: [aModelIndex entry asFileDirectory entries notEmpty]! !

!UiDirectoryModel methodsFor: 'accessing - data' stamp: 'mt 10/9/2010 16:37:59.356'!
iconFor: aModelIndex

	aModelIndex column = 1
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) isDirectory
			ifTrue: [UiSilkIcons folderIcon]
			ifFalse: [nil "UiSilkIcons pageWhiteIcon"]].
	
	aModelIndex column = 3
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) modificationDateAndTime year = 2010
			ifTrue: [UiSilkIcons newIcon]
			ifFalse: []].
		
	^ nil! !

!UiDirectoryModel methodsFor: 'accessing' stamp: 'mt 10/9/2010 20:36:01.933'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	^ (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex;
		setProperty: #entry toValue: ((aParentModelIndex
			ifNil: [self root]
			ifNotNil: [:index | (index valueOfProperty: #entry) asFileDirectory]) entries at: aRowNumber)! !

!UiDirectoryModel methodsFor: 'accessing' stamp: 'mt 10/7/2010 18:38:14.259'!
root

	^ FileDirectory on: 'D:'! !

!UiDirectoryModel methodsFor: 'accessing - data' stamp: 'mt 10/9/2010 16:34:45.73'!
textFor: aModelIndex

	aModelIndex column = 1
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) name asText].
		
	aModelIndex column = 2
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) fileSize asString asText].
		
	aModelIndex column = 3
		ifTrue: [^ (aModelIndex valueOfProperty: #entry) modificationDateAndTime asString asText].
		
	^ nil! !

!UiViewRowInfo methodsFor: 'accessing' stamp: 'mt 10/9/2010 18:20:34.641'!
height

	^ size ifNil: [size := Preferences standardListFont height]! !

!UiViewRowInfo methodsFor: 'geometry' stamp: 'mt 10/9/2010 18:20:50.9'!
optimalSize
	"Return the height of the largest item."
	
	| max |
	max := 0.
	items do: [:item | max := max max: item height].
	^ max! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/9/2010 17:19:28.005'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	| index |
	index := (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex.
		
	aParentModelIndex ifNil: [
		index setProperty: #package toValue: (PackageOrganizer default packages at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		index
			setProperty: #systemCategory
			toValue: ((aParentModelIndex valueOfProperty: #package) systemCategories at: aRowNumber).
		index
			setProperty: #systemCategoryPrefix
			toValue: (aParentModelIndex valueOfProperty: #package) systemCategoryPrefix.
		^ index].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		index
			setProperty: #class
			toValue: ((SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory))
				at: aRowNumber).
		^ index].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		class := Smalltalk at: (aParentModelIndex valueOfProperty: #class).
		index
			setProperty: #organization
			toValue: class organization.
		index
			setProperty: #protocol
			toValue: (class organization categories at: aRowNumber).
		^ index].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		index
			setProperty: #method
			toValue: ((organization listAtCategoryNamed: protocol) at: aRowNumber).
		^ index].! !

!UiSmalltalkModel methodsFor: 'accessing' stamp: 'mt 10/9/2010 17:31:54.663'!
rowCountFor: aParentModelIndex
	
	aParentModelIndex ifNil: [
		^ PackageOrganizer default packages size].
	
	aParentModelIndex column > 1 ifTrue: [^ 0].
	
	(aParentModelIndex includesProperty: #package) ifTrue: [
		^ (aParentModelIndex valueOfProperty: #package) systemCategories size].
	
	(aParentModelIndex includesProperty: #systemCategory) ifTrue: [
		^ (SystemOrganization
				listAtCategoryNamed: (aParentModelIndex valueOfProperty: #systemCategory)) size].
	
	(aParentModelIndex includesProperty: #class) ifTrue: [ | class |
		^ (Smalltalk at: (aParentModelIndex valueOfProperty: #class)) organization categories size].

	(aParentModelIndex includesProperty: #protocol) ifTrue: [ | organization protocol |
		organization := aParentModelIndex valueOfProperty: #organization.
		protocol := aParentModelIndex valueOfProperty: #protocol.
		^ (organization listAtCategoryNamed: protocol) size].
	
	^ 0! !

!UiSmalltalkModel methodsFor: 'accessing - data' stamp: 'mt 10/10/2010 11:06:01.969'!
textFor: aModelIndex

	| col package systemCategory class protocol method |
	col := aModelIndex column.
	package := aModelIndex valueOfProperty: #package ifAbsent: [].
	systemCategory := aModelIndex valueOfProperty: #systemCategory  ifAbsent: [].
	class := aModelIndex valueOfProperty: #class ifAbsent: [].
	protocol := aModelIndex valueOfProperty: #protocol ifAbsent: [].
	method := aModelIndex valueOfProperty: #method ifAbsent: [].
	
	package ifNotNil: [:p | 
		col = 1 ifTrue: [^ p name].
		col = 2 ifTrue: [^ ('<font color="#555555">', p systemCategories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">package</font>' asHtmlText]].
	
	systemCategory ifNotNil: [:cat | 
		| prefix |
		prefix := aModelIndex valueOfProperty: #systemCategoryPrefix.
		
		col = 2 ifTrue: [^ ('<font color="#555555">', (SystemOrganization listAtCategoryNamed: cat) size asString, '</font>') asHtmlText].
		
		cat = prefix ifFalse: [cat := cat allButFirst: prefix size + 1].
		col = 1 ifTrue: [^ cat].
		col = 3 ifTrue: [^ '<font color="#555555">system category</font>' asHtmlText]].
	
	class ifNotNil: [:c |
		col = 1 ifTrue: [^ c].
		col = 2 ifTrue: [^ ('<font color="#555555">', (Smalltalk at: c) organization categories size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">class</font>' asHtmlText]].

	protocol ifNotNil: [:p |
		| organization |
		organization := aModelIndex valueOfProperty: #organization.
		col = 1 ifTrue: [^ p].
		col = 2 ifTrue: [^ ('<font color="#555555">', (organization listAtCategoryNamed: p) size asString, '</font>') asHtmlText].
		col = 3 ifTrue: [^ '<font color="#555555">protocol</font>' asHtmlText]].
	
	method ifNotNil: [:m |
		col = 1 ifTrue: [^ '#', m].
		col = 3 ifTrue: [^ '<font color="#555555">method</font>' asHtmlText]].
		
	^ nil! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 10/9/2010 19:51:16.221'!
optimalSize
	"Return the width of the largest item."
	
	| max |
	max := 0.
	items do: [:item | max := max max: (item width + (item valueOfProperty: #indent))].
	^ max! !

!UiViewColumnInfo methodsFor: 'accessing' stamp: 'mt 10/9/2010 18:15:17.434'!
width

	^ size ifNil: [^ 150]! !
